// Database connection name
global string g_unLHCLoggingDBSql_connectionName; 

// Database connection
global dbConnection g_unLHCLoggingDBSql_dbConnection; 


// Connect to a specified DB, if the connectionName exists, then use the existing. If connexction fails return FALSE.
public bool unLHCLoggingDBSql_openDbConnection(string sDbName, string sDbUser, string sDbPass)
{ 
  // Close connection first, in case if it already exists
  unLHCLoggingDBSql_closeDbConnection(); 
  
  // Specify new connection name
  g_unLHCLoggingDBSql_connectionName = "RDBLHCLogging";  
  
  string errTxt;
  int rc=fwDbGetConnection(g_unLHCLoggingDBSql_connectionName,g_unLHCLoggingDBSql_dbConnection);
  if (rc==-1){
    fwDbCheckError(errTxt,g_unLHCLoggingDBSql_dbConnection);
    DebugN("ERROR Getting the connection",errTxt);
    return false;
  } 
  else if (rc==1) { // connection with such name does not exist yet. Create it
    string connectString = "Driver=QOCI;Database=" + sDbName + ";User=" + sDbUser + ";ENC_PWD=" + sDbPass;
    fwDbOpenConnection(connectString,g_unLHCLoggingDBSql_dbConnection,g_unLHCLoggingDBSql_connectionName);
    if (fwDbCheckError(errTxt,g_unLHCLoggingDBSql_dbConnection)){DebugN("ERROR WHILE OPENING DATABASE",errTxt);return false;};
  }
  //Add a debug info also for existed connection?

  mixed retVal; // note! the argument to fwDbOption must be of type mixed!
  int rc=fwDbOption("Inspect",g_unLHCLoggingDBSql_dbConnection,retVal);
  DebugN(rc,retVal);        
  return true;
}


// Close database connection
public void unLHCLoggingDBSql_closeDbConnection()
{  
  // Close connection only if it already exists to avoid error caused by too many open connections
  if(fwDbGetConnection(g_unLHCLoggingDBSql_connectionName,g_unLHCLoggingDBSql_dbConnection) == 0 )   
  {
    fwDbDeleteConnection(g_unLHCLoggingDBSql_connectionName); 
  }
  // TODO: maybe this one is better:  reset the connection manager - terminate all connections that are still present
  //fwDbOption("Reset",0);   
}


// Execute DB query
public dyn_dyn_mixed unLHCLoggingDBSql_queryData(string sqlQuery, dyn_string &dsExInfo, bool columnWise=false)
{
  dyn_dyn_mixed data;    
  dbCommand cmd;
  string errTxt;  
  
  fwDbStartCommand(g_unLHCLoggingDBSql_dbConnection, sqlQuery, cmd);  
  if (fwDbCheckError(errTxt,cmd)) {fwException_raise(dsExInfo,"DB ERROR", errTxt, ""); return data; };

  fwDbExecuteCommand(cmd);
  if (fwDbCheckError(errTxt,cmd)) {fwException_raise(dsExInfo,"DB ERROR", errTxt, ""); return data; };

  fwDbGetData(cmd, data, columnWise);
  if (fwDbCheckError(errTxt,cmd)) {fwException_raise(dsExInfo,"DB ERROR", errTxt, ""); return data; }; 

  fwDbFinishCommand(cmd);
  if (fwDbCheckError(errTxt)){fwException_raise(dsExInfo,"DB ERROR", errTxt, ""); return data; }; 
    
  return data;  
}


// Executes complex queries from DB, in case of fail returns false.
public dyn_dyn_mixed unLHCLoggingDBSql_queryDataWithParMapping(string sqlQuery, mapping params, dyn_string &dsExInfo, bool columnWise=false)
{
  string errTxt;
  dbCommand cmd;
  dyn_dyn_mixed data;
  
  fwDbStartCommand(g_unLHCLoggingDBSql_dbConnection, sqlQuery, cmd);
  if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt); return data;};    
    
  if (mappinglen(params)) {
     fwDbBindParams(cmd, params);
     if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt); return data;};
   }
    
  fwDbExecuteCommand(cmd);
  if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt); return data;};
  
  fwDbGetData(cmd, data, columnWise);
  if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt); return data;};

  fwDbFinishCommand(cmd);
  if (fwDbCheckError(errTxt)){DebugN("DB ERROR",errTxt); return data;}; 
    
  return data;  
} 


/**
* Insert bound data into database
*
* @param sql       input, SQL command string
* @param data      input, data to be inserted    

* @return          output, true: success, false: error
*/
public bool unLHCLoggingDBSql_insertBoundData(string sql, dyn_dyn_mixed &data )
{    
  // Insert data records into table GTT_STAGE_VARIABLES     
  dbCommand cmd;
  string errTxt;
  
  fwDbStartCommand(g_unLHCLoggingDBSql_dbConnection,sql, cmd);
  if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt);return false;};  
 
  fwDbBindAndExecute(cmd, data);
  if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt);return false;};  
 
  fwDbFinishCommand(cmd);   
  if (fwDbCheckError(errTxt)){DebugN("DB ERROR",errTxt);return false;}; 

  return true;
}


/**
* Execute sql command with parameter mapping
*
* @param sql       input, SQL command string
* @param params    input, Parameter papping like: param["user"]=sUserName;      

* @return          output, true: success, false: error
*/
public bool unLHCLoggingDBSql_executeWithParMapping(string sql, mapping params)
{
    string errTxt;
    dbCommand cmd;
  
    fwDbStartCommand(g_unLHCLoggingDBSql_dbConnection, sql, cmd);
    if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt);return false;};    
    
    if (mappinglen(params)) {
       fwDbBindParams(cmd, params);
       if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt);return false;};
     }
    
    fwDbExecuteCommand(cmd);
    if (fwDbCheckError(errTxt,cmd)){DebugN("DB ERROR",errTxt);return false;};

    fwDbFinishCommand(cmd);
    if (fwDbCheckError(errTxt)){DebugN("DB ERROR",errTxt);return false;}; 
    
    return true;  
}


/**
* Get all DB schemas, which have an ELEMENTS table
*
* @param   dsExInfo      output: Exeption info   
* @return  dsDbSchemas   output: List of DB schemas                       
*/
public dyn_string unLHCLoggingDBSql_getDbSchemas(dyn_string &dsExInfo)
{  
  dyn_string dsDbSchemas;    
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData("select OWNER from all_tables where table_name='ELEMENTS' order by OWNER asc", dsExInfo);
  if(dynlen(dsExInfo)==0) {
    if(dynlen(sqlResult) > 0) { 
      dsDbSchemas = getDynMixed(sqlResult,1);  //Get only column 1     
      dynAppend(dsDbSchemas, "*");  // add star to select all tables 
    } 
    else {
      fwException_raise(dsExInfo,"DB ERROR", "No database schemas found", "");
    }  
  }  
  return dsDbSchemas;  
}


/**
* Get all elements registered for logging of a specified database schema 
*
* @param   sDbSchema    input,  Database schema
* @param   bUTC         input,  true=show timestamps as UTC, false=show timestamps as localtime
* @param   dsExInfo     output: Exeption info  
*
* @return  sqlResult    output, Query result (column wise)
*
* sqlResult[1][x]: Variable ID
* sqlResult[2][x]: Variable Name
* sqlResult[3][x]: Dpe NAME
* sqlResult[4][x]: Dpe ALIAS
* sqlResult[5][x]: Data category
* sqlResult[6][x]: User name
* sqlResult[7][x]: Logging required
* sqlResult[8][x]: Registered for logging
* sqlResult[9][x]: Rdb source
* sqlResult[10][x]: Hierarchy
* sqlResult[11][x]: Transfer job ID
* sqlResult[12][x]: Timestamp last job run
* sqlResult[13][x]: Timestamp last logged
* sqlResult[14][x]: No records last logged
*
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getLogVarInfo(string sDbSchema, bool bUTC=false, dyn_string &dsExInfo)
{
  string sql = "select TO_CHAR(mv.VARIABLE_ID), " +
               " mv.VARIABLE_NAME, " +    
               " el.ELEMENT_NAME, " + 
               " el.ALIAS, " + 
               " mvc.DATA_CATEGORY_NAME || '/' || mvc.DATA_CATEGORY_SUB_NAME, " +
               " mu.REAL_NAME, " +                       
               " replace(replace(mv.LOGGING_REQUIRED,'Y','YES'),'N','NO'), " +  
               " replace(replace(mv.REGISTERED_FOR_LOGGING,'Y','YES'),'N','NO'), " + 
               " mv.RDB_SOURCE, " +                      
               " replace(mh.NODE_PATH,'->','/'), " +
               " mvc.DATA_TRANSFER_GROUP_ID, " + 
               " to_char(" + ( bUTC==true ? "lvs.CREATED_UTC_STAMP" : "from_tz(lvs.CREATED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +
               " to_char(" + ( bUTC==true ? "lvs.LAST_LOGGED_UTC_STAMP" : "from_tz(lvs.LAST_LOGGED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +
               " lvs.no_recs_logged "                 
	              "from RDB2LHC.META_VARIABLES mv inner join " + sDbSchema + ".ELEMENTS el on mv.ELEMENT_ID = el.ELEMENT_ID " +
	              " left outer join RDB2LHC.META_VARIABLE_CATEGORIES mvc on mv.DATA_CATEGORY_ID = mvc.DATA_CATEGORY_ID " +
	              " left outer join RDB2LHC.META_USERS mu on mv.OWNER = mu.USER_NAME " +
	              " left outer join RDB2LHC.META_HIERARCHY_VARIABLES mhv on mv.VARIABLE_ID = mhv.VARIABLE_ID " +
	              " left outer join RDB2LHC.META_HIERARCHIES mh on mhv.HIERARCHY_ID = mh.HIERARCHY_ID " +                     
               " left outer join RDB2LHC.LOG_VAR_STATUS_MAX lvsm on mvc.data_transfer_group_id = lvsm.data_transfer_group_id " + 
               " left outer join RDB2LHC.LOG_VARIABLE_STATUS lvs on mv.variable_id = lvs.variable_id and mvc.data_transfer_group_id = lvs.data_transfer_group_id and lvsm.created_utc_stamp = lvs.created_utc_stamp ";  
      
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo, true);
  return sqlResult;  
} 


/**
* Get details of a particular element 
* 
* Use this function only to query single elements because it is not
* very fast, which is caused by the prequery of the dbSchema name
*
* @param   sVariableId   input,  Variable ID of the selected element
* @param   dsExInfo      output, ExceptionInfo

* @return  sqlResult      output, Result of the query (row wise)
*
* sqlResult[1][1]: Variable name
* sqlResult[1][2]: Datapoint name
* sqlResult[1][3]: Data category
* sqlResult[1][4]: User name
* sqlResult[1][5]: Logging requred
* sqlResult[1][6]: Registered for logging
* sqlResult[1][7]: Rdb source
* sqlResult[1][8]: Hierarchy
*
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getElementDetails(string sVariableId, dyn_string &dsExInfo)
{  
  mapping params;
  params["VarId"] = sVariableId;    
  
  string sql1 = "select SUBSTR(RDB_SOURCE, 1, INSTR4(RDB_SOURCE, '.')-1) from RDB2LHC.META_VARIABLES where VARIABLE_ID = :VarId";        
    
  // Get name of RDB schema    
  dyn_dyn_mixed sqlResult1 = unLHCLoggingDBSql_queryDataWithParMapping(sql1, params, dsExInfo);  
  string sDbSchema = sqlResult1[1][1];
  
  string sql2 = "select mv.VARIABLE_NAME, " +    
                 "el.ELEMENT_NAME, " + 
                 "mvc.DATA_CATEGORY_NAME || '/' || mvc.DATA_CATEGORY_SUB_NAME, " +
                 "mu.REAL_NAME, " +                       
                 "replace(replace(mv.LOGGING_REQUIRED,'Y','1'),'N','0'), " +  
                 "replace(replace(mv.REGISTERED_FOR_LOGGING,'Y','1'),'N','0'), " + 
                 "mv.RDB_SOURCE, " +                      
                 "replace(mh.NODE_PATH,'->','/') " +  
	              "from RDB2LHC.META_VARIABLES mv inner join " + sDbSchema + ".ELEMENTS el on mv.ELEMENT_ID = el.ELEMENT_ID " +
	                "left outer join RDB2LHC.META_VARIABLE_CATEGORIES mvc on mv.DATA_CATEGORY_ID = mvc.DATA_CATEGORY_ID " +
	                "left outer join RDB2LHC.META_USERS mu on mv.OWNER = mu.USER_NAME " +
	                "left outer join RDB2LHC.META_HIERARCHY_VARIABLES mhv on mv.VARIABLE_ID = mhv.VARIABLE_ID " +
	                "left outer join RDB2LHC.META_HIERARCHIES mh on mhv.HIERARCHY_ID = mh.HIERARCHY_ID " + 
               "where mv.VARIABLE_ID = :VarId";     
  
  // Get data line wise  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql2, params, dsExInfo);

  return sqlResult;  
} 


/**
* Get the transfer history of a specific element (MDB -> LDB)
*
* @param   sVariableId   input,  Variable ID of the selected element
* @param   tStart        input,  Query start time (UTC only)
* @param   tEnd          input,  Query end time (UTC only)
* @param   bUTC          input,  Display timestamps of query result in UTC or LOCAL
* @param   dsExInfo      output, ExceptionInfo

* @return  sqlResult      output, Result of the query (column wise)
*
* sqlResult[1][x]: Time created
* sqlResult[2][x]: Time first checked
* sqlResult[3][x]: Time last checked
* sqlResult[4][x]: No of records checked
* sqlResult[5][x]: Time last logged
* sqlResult[6][x]: No of records logged
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getTransferHistory(string sVariableId, time tStart, time tEnd, bool bUTC=false, dyn_string &dsExInfo)
{
   string sql = "select to_char(" + ( bUTC==true ? "CREATED_UTC_STAMP" : "from_tz(CREATED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +
               " to_char(" + ( bUTC==true ? "FIRST_CHECKED_UTC_STAMP" : "from_tz(FIRST_CHECKED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +
               " to_char(" + ( bUTC==true ? "LAST_CHECKED_UTC_STAMP" : "from_tz(LAST_CHECKED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +
               " NO_RECS_CHECKED, " + 
               " to_char(" + ( bUTC==true ? "LAST_LOGGED_UTC_STAMP" : "from_tz(LAST_LOGGED_UTC_STAMP, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), " +                
               " NO_RECS_LOGGED " + 	                 
	              "from RDB2LHC.LOG_VARIABLE_STATUS " +
               " where variable_id = :varId and CREATED_UTC_STAMP BETWEEN :t1 and :t2 " +
	              " order by created_utc_stamp desc"; 
 
  mapping params;
  params[":varId"]=sVariableId;
  params[":t1"]=tStart;
  params[":t2"]=tEnd;    

  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, dsExInfo, true);
  
  return sqlResult;  
}


/**
* Get value history of a specific meta variable 
*
* @param   sVariableId  input,  VARIABLE_ID of the selected element
* @param   tStart       input,  Query start time (UTC only)
* @param   tEnd         input,  Query end time (UTC only)
* @param   bUTC         input,  Display timestamps of query result in UTC or LOCAL
* @param   dsExInfo     output, ExceptionInfo

* @return  sqlResult    output, Result of the query (column wise)
*
* sqlResult[1][x]: Time archived to MDB
* sqlResult[2][x]: Value
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getElementHistory(string sVariableId, time tStart, time tEnd, bool bUTC=false, dyn_string &dsExInfo)
{
  // Prequery for RDB_SOURCE and ELEMENT_ID
  mapping params1;
  params1["VarId"] = sVariableId;   
  
  string sql1 = "SELECT rdb_source " + 
                "     , TO_CHAR(element_id) " + 
                "FROM rdb2lhc.meta_variables " + 
                "WHERE variable_id = :VarId" ;  
  
  dyn_dyn_mixed sqlResult1 = unLHCLoggingDBSql_queryDataWithParMapping(sql1, params1, dsExInfo);  
  string sRdbSource = sqlResult1[1][1]; 
  string sElementId = sqlResult1[1][2]; 
  
  
  // Main query for history data
  mapping params2;
  params2[":elementId"]=sElementId;
  params2[":t1"]=tStart;
  params2[":t2"]=tEnd; 
  
  string sql2 = "select to_char(" + ( bUTC==true ? "TS" : "from_tz(TS, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), "+
               " TO_NUMBER(VALUE_NUMBER) " +
               "from " + sRdbSource + 
               " where ELEMENT_ID = :elementId and TS between :t1 and :t2" +
               " order by TS desc";   
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql2, params2, dsExInfo, true);
  
  return sqlResult;  
}


/**
* Get meta users from RDB2LHC.META_USERS
*
* @param dsUserNames    output, list of user names
* @param dsRealNames    output, list of real names
* @param dsExInfo       output, Exeption information
*/
public void unLHCLoggingDBSql_getMetaUsers(dyn_string &dsUserNames, dyn_string &dsRealNames, dyn_string &dsExInfo)
{  
  dynClear(dsUserNames);
  dynClear(dsRealNames);
  
  string sql = "SELECT USER_NAME, REAL_NAME from RDB2LHC.META_USERS";  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo, true);  
  
  if(dynlen(dsExInfo)==0) {  
    dsUserNames = sqlResult[1];
    dsRealNames = sqlResult[2];
  }
}


/**
* Check if user pass combination is found in the database
*
* @param sUser    output, username
* @param sPass    output, password

* @return         output, true: success, false: error
*/
public bool unLHCLoggingDBSql_checkUserPass(string sUser, string sPass)
{  
  dyn_string exceptionInfo;
  
  string sql = "select USER_NAME from RDB2LHC.META_USERS where USER_NAME = :username and PASSWORD = :password";
  
  mapping params;
  params[":username"] = sUser;
  params[":password"] = sPass;    
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, exceptionInfo);  
                            
  if(dynlen(sqlResult)==1) {
    if(sqlResult[1][1]==sUser) {
      return true;  
    }
  }  
  
  return false;
}


/**
* Get password for a known username
*
* @param sUser    input, username

* @return         output, password
*/
public string unLHCLoggingDBSql_getPassword(string sUser)
{  
  dyn_string exceptionInfo;
  
  string sql = "select PASSWORD from RDB2LHC.META_USERS where USER_NAME = :username";
  
  mapping params;
  params[":username"] = sUser;
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, exceptionInfo);  
                            
  if(dynlen(sqlResult)==1) {
    return sqlResult[1][1];
  }  
  
  return ("");
}


/**
* Get data categories from RDB2LHC.META_VARIABLE_CATEGORIES
*
* @param dsDataCategoryIds    output, list of data category ids
* @param dsDataCategoryNames  output, list of data category names
* @param dsExInfo               output, Exeption information
*/
public void unLHCLoggingDBSql_getDataCategories(dyn_string &dsDataCategoryIds, dyn_string &dsDataCategoryNames, dyn_string &dsExInfo)
{  
  dynClear(dsDataCategoryIds);
  dynClear(dsDataCategoryNames);
  
  string sql = "SELECT DATA_CATEGORY_ID, DATA_CATEGORY_NAME, DATA_CATEGORY_SUB_NAME from RDB2LHC.META_VARIABLE_CATEGORIES";  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo);  
  
  for(int i=1; i<=dynlen(sqlResult);i++) { 
    dsDataCategoryIds[i] = sqlResult[i][1];
    dsDataCategoryNames[i] = sqlResult[i][2] + "/" + sqlResult[i][3];
  } 
}


/**
* Execute SQL query to get session types
*
* @param    dsExInfo           output, Exeption information   
* @return   dsSessionTypes     output, list of session types
*/
public dyn_string unLHCLoggingDBSql_getSessionTypes(dyn_string &dsExInfo)
{
  dyn_string dsSessionTypes; 
  
  string sql = "select distinct SESSION_TYPE from RDB2LHC.audit_session " +
               "order by regexp_substr(SESSION_TYPE, '^\\D*') nulls first, to_number(regexp_substr(SESSION_TYPE, '\\d+'))";  
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo, true);

  if(dynlen(dsExInfo)==0) {
    if(dynlen(sqlResult) > 0) {
      dsSessionTypes = sqlResult[1];
    }
  } 
  return dsSessionTypes;  
}


/**
* Execute sql command with parameter mapping
*
* @param dsElementId       input, list of element ids
* @param dsVariableName    input, list of variable names
* @param dsRDBSource       input, list of RDB sources
* @param dsUnit            input, list of units
* @param dsDescription     input, list of descriptions
* @param sDataCategoryId   input, list of data category ids
* @param dsHierarchy       input, list of Hierarchies
* @param sUser             input, username from meta_users
* @param sPass             input, password from meta_users

* @return                  output, true: success, false: error
*/
public bool unLHCLoggingDBSql_registerDataForLogging(const dyn_string &dsElementId, const dyn_string &dsVariableName, const dyn_string &dsRDBSource, const dyn_string &dsUnit, 
                            const dyn_string &dsDescription, const string &sDataCategoryId, const dyn_string &dsHierarchy, const string sUser, const string sPass)
{
  dyn_dyn_mixed ddmVariablesToInsert;
  dyn_dyn_mixed ddmHierarchiesToInsert; 
    
  // Prepare Values to be inserted into the DB
  int numberOfElements = dynlen(dsElementId);
  for(int i=1 ; i <= numberOfElements; i++)
  { 
    // Animate progress bar    
    progressBar.progress(i * 100 / numberOfElements);  // Progress from 0 to 100 %  
    
    dynAppend(ddmVariablesToInsert, makeDynMixed(dsElementId[i], dsVariableName[i], dsRDBSource[i], dsUnit[i], dsDescription[i], sDataCategoryId)); 
    
    if (dsHierarchy[i] != "") {
      dynAppend(ddmHierarchiesToInsert, makeDynMixed(dsElementId[i], dsHierarchy[i]));
    }  
  }  
  
  // Insert into GTT_STAGE_VARIABLES 
  if(fwDbExecuteSqlStatement(g_unLHCLoggingDBSql_dbConnection, "delete from RDB2LHC.GTT_STAGE_VARIABLES") !=0 ) {  
    DebugTN("Delete error"); 
    return false;  
  }   
  string sql = "INSERT INTO RDB2LHC.GTT_STAGE_VARIABLES(ELEMENT_ID,VARIABLE_NAME,RDB_SOURCE,UNIT,DESCRIPTION,DATA_CATEGORY_ID) VALUES(:data1,:data2,:data3,:data4,:data5,:data6)";  
  if(unLHCLoggingDBSql_insertBoundData(sql, ddmVariablesToInsert) == false) {
    DebugTN("Inserting values into GTT_STAGE_VARIABLES failed"); 
    return false;  
  }   
  
  // Insert into GTT_STAGE_HIERARCHIES
  if(dynlen(ddmHierarchiesToInsert) > 0)  
  {
    if(fwDbExecuteSqlStatement(g_unLHCLoggingDBSql_dbConnection, "delete from RDB2LHC.GTT_STAGE_HIERARCHIES") !=0 ) {  
      DebugTN("Delete error"); 
      return false;  
    } 
    string sql = "INSERT INTO RDB2LHC.GTT_STAGE_HIERARCHIES(ELEMENT_ID,HIERARCHY_PATH) VALUES(:data1,replace(:data2,'/','->'))";    
    if(unLHCLoggingDBSql_insertBoundData(sql, ddmHierarchiesToInsert) == false) {
      DebugTN("Inserting values into GTT_STAGE_HIERARCHIES failed"); 
      return false;  
    }
  }
  
  // Register metadata
  mapping params;
  params[":username"] = sUser;
  params[":password"] = sPass;    
  string sql = "begin pvss_data_transfer_mgr.register_metadata(:username,:password); end;" ;  
  if(unLHCLoggingDBSql_executeWithParMapping(sql,params) == false) {
    DebugTN("Registering Data Failed"); 
    return false;  
  }   
  
  return true;  
}


/*
  This will be the new function, the other one will be deleted later
*/  
public bool unLHCLoggingDBSql_registerDataForLoggingNew(const dyn_string &dsElementId, const dyn_string &dsVariableName, const dyn_string &dsRDBSource, const dyn_string &dsUnit, 
                            const dyn_string &dsDescription, const dyn_string &dsDataCategoryId, const dyn_string &dsHierarchy, const string sUser, const string sPass, bool bEnableProgressBar=true)
{
  dyn_dyn_mixed ddmVariablesToInsert;
  dyn_dyn_mixed ddmHierarchiesToInsert; 
    
  // Prepare Values to be inserted into the DB
  int numberOfElements = dynlen(dsElementId);
  for(int i=1; i<=numberOfElements; i++) 
  {     
    // Animate progress bar    
    if (bEnableProgressBar==true) 
      progressBar.progress(i * 100 / numberOfElements);  
   
    // prepare meta_variables
    dynAppend(ddmVariablesToInsert, makeDynMixed(dsElementId[i], dsVariableName[i], dsRDBSource[i], dsUnit[i], dsDescription[i], dsDataCategoryId[i])); 
   
    // prepare meta_hierarchies
    if (dsHierarchy[i] != "")
      dynAppend(ddmHierarchiesToInsert, makeDynMixed(dsElementId[i], dsHierarchy[i]));      
  } 
 
  // Insert into GTT_STAGE_VARIABLES 
  if(fwDbExecuteSqlStatement(g_unLHCLoggingDBSql_dbConnection, "delete from RDB2LHC.GTT_STAGE_VARIABLES") !=0 ) {  
    DebugTN("Delete error"); 
    return false;  
  }   
  string sql = "INSERT INTO RDB2LHC.GTT_STAGE_VARIABLES(ELEMENT_ID,VARIABLE_NAME,RDB_SOURCE,UNIT,DESCRIPTION,DATA_CATEGORY_ID) VALUES(:data1,:data2,:data3,:data4,:data5,:data6)";  
  if(unLHCLoggingDBSql_insertBoundData(sql, ddmVariablesToInsert) == false) {
    DebugTN("Inserting values into GTT_STAGE_VARIABLES failed"); 
    return false;  
  }   
  
  // Insert into GTT_STAGE_HIERARCHIES
  if(dynlen(ddmHierarchiesToInsert) > 0)  
  {
    if(fwDbExecuteSqlStatement(g_unLHCLoggingDBSql_dbConnection, "delete from RDB2LHC.GTT_STAGE_HIERARCHIES") !=0 ) {  
      DebugTN("Delete error"); 
      return false;  
    } 
    string sql = "INSERT INTO RDB2LHC.GTT_STAGE_HIERARCHIES(ELEMENT_ID,HIERARCHY_PATH) VALUES(:data1,replace(:data2,'/','->'))";    
    if(unLHCLoggingDBSql_insertBoundData(sql, ddmHierarchiesToInsert) == false) {
      DebugTN("Inserting values into GTT_STAGE_HIERARCHIES failed"); 
      return false;  
    }
  }
  
  // Register metadata
  mapping params;
  params[":username"] = sUser;
  params[":password"] = sPass;    
  string sql = "begin pvss_data_transfer_mgr.register_metadata(:username,:password); end;" ;  
  if(unLHCLoggingDBSql_executeWithParMapping(sql,params) == false) {
    DebugTN("Registering Data Failed"); 
    return false;  
  }   
  
  return true;  
}


/**
* Query current database system time 
*
* @param dsExInfo     output, Exeption information   

* @return  sqlResult  output, database systimestamp as UTC
*/
public time unLHCLoggingDBSql_getDbSysTimeUtc(dyn_string &dsExInfo)
{  
  string sql = "SELECT SYS_EXTRACT_UTC(SYSTIMESTAMP) FROM DUAL";  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo);  
  
  time t=0;
  if(dynlen(sqlResult)==1) 
    t = sqlResult[1][1];  
  return t;
}


/**
* Query data of actual logging status
*
* @param sRdbSource   input, RDB source of this system
* @param dsExInfo     output, Exeption information   

* @return  sqlResult  output, Result of the query (row wise)
*
* sqlResult[x][1]: Variable name
* sqlResult[x][2]: CREATED_UTC_STAMP
* sqlResult[x][3]: LAST_LOGGED_UTC_STAMP
* sqlResult[x][4]: NO_RECS_LOGGED
* sqlResult[x][5]: MAX_ROWS
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getLvsInfo(string sRdbSource, dyn_string &dsExInfo)
{  
  string sql = "select mv.variable_name, "
               " lvs.created_utc_stamp, "
               " lvs.last_logged_utc_stamp, "  
               " lvs.no_recs_logged, " 
               " adtg.max_rows "
               "from RDB2LHC.META_VARIABLES mv "
               " left outer join RDB2LHC.META_VARIABLE_CATEGORIES mvc on mv.DATA_CATEGORY_ID = mvc.DATA_CATEGORY_ID "                    
               " left outer join RDB2LHC.LOG_VAR_STATUS_MAX lvsm on mvc.data_transfer_group_id = lvsm.data_transfer_group_id "  
               " left outer join RDB2LHC.LOG_VARIABLE_STATUS lvs on mv.variable_id = lvs.variable_id and mvc.data_transfer_group_id = lvs.data_transfer_group_id and lvsm.created_utc_stamp = lvs.created_utc_stamp "  
               " left outer join RDB2LHC.ADMIN_DATA_TRANSFER_GROUPS adtg on mvc.data_transfer_group_id = adtg.data_transfer_group_id "
               "where mv.rdb_source like :RdbSource ";
  
  mapping params;
  params[":RdbSource"]=sRdbSource;
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, dsExInfo);
  
  return sqlResult;
}


/**
* Query logging status for a specific test-datapoint, 
* get the 3 most recent entries in LVS status within the last hour. 
*
* @param sVarName       input, Variable Name of the test datapoint
* @param dsExInfo       output, Exeption information   

* @return  sqlResult    output, Result of the query (row wise)
*
* sqlResult[1][1]: lvs.created_utc_stamp (most recent)
* sqlResult[2][1]: lvs.created_utc_stamp (2nd most recent)
* sqlResult[3][1]: lvs.created_utc_stamp (3rd most recent)
* sqlResult[1][2]: lvs.last_logged_utc_stamp (most recent)
* sqlResult[2][2]: lvs.last_logged_utc_stamp (2nd most recent)
* sqlResult[3][2]: lvs.last_logged_utc_stamp (3rd most recent)
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getLvsForTestDpe(string sVarName, dyn_string &dsExInfo)
{    
  string sql = "SELECT * FROM ( " +
               "  SELECT lvs.created_utc_stamp " +
               "       , lvs.last_logged_utc_stamp " +
               "  FROM rdb2lhc.meta_variables mv JOIN rdb2lhc.log_variable_status lvs " +
               "    ON mv.variable_id = lvs.variable_id " +
               "  WHERE lvs.created_utc_stamp BETWEEN (SYS_EXTRACT_UTC(SYSTIMESTAMP)-1/24) " +
               "    AND SYS_EXTRACT_UTC(SYSTIMESTAMP) " +
               "    AND mv.variable_name=UPPER(:VarName) " +
               "  ORDER BY lvs.created_utc_stamp DESC " +
               ") " +
               "WHERE ROWNUM <=3 "; 
 
  mapping params;
  params[":VarName"]=sVarName;  
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, dsExInfo);
  
  return sqlResult;
}


/**
* Query for transfer statistics per transfer job
*
* @param job           input, name of the transfer job
* @param tStart        input, utc start time of time window 
* @param tEnd          input, utc end time of time window
* @param bUTC          input, true=show timestamps as UTC, false=show timestamps as localtime
* @param dsExInfo      output, exeption information   

* @return sqlResult    output, Result of the query (column wise)
*
* sqlResult[1]: session_id
* sqlResult[2]: session_start_utc
* sqlResult[3]: session_type
* sqlResult[4]: time_in_s
* sqlResult[5]: no_of_recs
* sqlResult[6]: time_in_s
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getTransferStats(string job, time tStart, time tEnd, bool bUTC, dyn_string &exceptionInfo)
{
  string sql;  
  sql="select ao1.session_id, " + 
      ( bUTC==true ? " from_tz(s.session_start_utc, 'UTC') at local, " : " s.session_start_utc, " ) + 
      " s.session_type, " +
      " ao1.time_in_s t1, "+
      " ao1.no_of_recs/1000 nrec1k, "+
      " ao2.time_in_s t2, " + 
      " ao2.no_of_recs/1000 nrec2k " +
      "from audit_operation ao1 " +
      " join audit_operation ao2 on ao1.session_id=ao2.session_id" +
      " left outer join audit_session s on s.session_id=ao1.session_id " +
      " where ao1.op_type like 'COLLECT_%' and ao2.op_type like 'LOG%' " +
      " and session_type=:transferid " + 
      "AND SESSION_START_UTC BETWEEN :t1 and :t2 " +   
      " order by session_id desc ";    
  
  mapping params;
  params[":transferid"]=job;
  params[":t1"]=tStart;
  params[":t2"]=tEnd;
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, exceptionInfo, true);
  return sqlResult;  
}

// data[1]: session_id
// data[2]: session_start_utc
// data[3]: session_type
// data[4]: time_in_s
// data[5]: no_of_recs
// data[6]: time_in_s
public dyn_dyn_mixed unLHCLoggingDBSql_getTransferStatsForPerformancePanel(int job, time tStart, time tEnd, dyn_string &exceptionInfo)
{
  string sql="select ao1.session_id, s.session_start_utc, s.session_type, " +
             "   ao1.time_in_s t1, ao1.no_of_recs/1000 nrec1k, ao2.time_in_s t2, ao2.no_of_recs/1000 nrec2k " +
             "from audit_operation ao1 " +
             " join audit_operation ao2 on ao1.session_id=ao2.session_id" +
             " left outer join audit_session s on s.session_id=ao1.session_id " +
             " where ao1.op_type like 'COLLECT_%' and ao2.op_type like 'LOG%' " +
             " and session_type=:transferid " + 
             "AND SESSION_START_UTC BETWEEN :t1 and :t2 " +   
             " order by session_id desc ";   
  
  mapping params;
  params[":transferid"]="LDB_DATA_TRANSFER_"+job;
  params[":t1"]=tStart;
  params[":t2"]=tEnd;
  
  dyn_dyn_mixed data = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, exceptionInfo, true);
 
  return data;  
}

/**
* Execute SQL query to get information of RDB transfer jobs
*
* @param   sRdbSource         input, show transfer jobs related to this RDB SOURCE 
* @param   bUTC               input, true=show timestamps as UTC, false=show timestamps as localtime
* @param   dsExInfo           output, exeption information   

* @return  sqlResult          output, Result of the query (column wise)
*
* sqlResult[1]: job name
* sqlResult[2]: job enabled
* sqlResult[3]: job state
* sqlResult[4]: last start date
* sqlResult[5]: last run duration
* sqlResult[6]: next run date
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getTransferJobDetails(string sRdbSource, bool bUTC, dyn_string &dsExInfo)
{  
  if(sRdbSource == "") {
    sRdbSource = "%";  // show all jobs  
  }
  
  string sql;   
  sql = "select JOB_NAME, ENABLED, STATE, " 
      " to_char(" + ( bUTC==true ? "SYS_EXTRACT_UTC(LAST_START_DATE)" : "LAST_START_DATE" ) + ",'YYYY/MM/DD HH24:MI:SS'), " 
      " 3600*extract(hour from LAST_RUN_DURATION) + 60*extract(minute from LAST_RUN_DURATION) + extract(second from LAST_RUN_DURATION), " 
      " to_char(" + ( bUTC==true ? "SYS_EXTRACT_UTC(NEXT_RUN_DATE)" : "NEXT_RUN_DATE" ) + ",'YYYY/MM/DD HH24:MI:SS') "         
      "from DBA_SCHEDULER_JOBS where OWNER='RDB2LHC' and job_action like 'ldb_data_transfer_service.transfer_data_to_ldb%' " 
      " and job_action in ( "
      "  select distinct 'ldb_data_transfer_service.transfer_data_to_ldb(' || mvc.data_transfer_group_id || ');' from RDB2LHC.META_VARIABLES mv 	"	   
      "   left join RDB2LHC.META_VARIABLE_CATEGORIES mvc on mv.DATA_CATEGORY_ID = mvc.DATA_CATEGORY_ID "
      "  where RDB_SOURCE like '" + sRdbSource + "' "
      " ) "
      "order by regexp_substr(JOB_NAME, '^\\D*') nulls first, to_number(regexp_substr(JOB_NAME, '\\d+')) "; 
 
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo, true);

  return sqlResult; 
}


/**
* SQL query to get current status of transfer jobs
*
* @param   sRdbSource         input, show transfer jobs related to this RDB SOURCE 
* @param   dsExInfo           output, exeption information   

* @return  sqlResult          output, Result of the query 
*
* sqlResult[1]: job name
* sqlResult[2]: job enabled
* sqlResult[3]: job state
* sqlResult[4]: last start date (utc)
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getTransferJobStatus(string sRdbSource, dyn_string &dsExInfo)
{   
  string sql;   
  sql = "select JOB_NAME, ENABLED, STATE, SYS_EXTRACT_UTC(LAST_START_DATE) "        
        "from DBA_SCHEDULER_JOBS where OWNER='RDB2LHC' and job_action like 'ldb_data_transfer_service.transfer_data_to_ldb%' " 
        " and job_action in ( "
        "  select distinct 'ldb_data_transfer_service.transfer_data_to_ldb(' || mvc.data_transfer_group_id || ');' from RDB2LHC.META_VARIABLES mv 	"	   
        "   left join RDB2LHC.META_VARIABLE_CATEGORIES mvc on mv.DATA_CATEGORY_ID = mvc.DATA_CATEGORY_ID "
        "  where RDB_SOURCE like :RdbSource "
        " ) "
        "order by regexp_substr(JOB_NAME, '^\\D*') nulls first, to_number(regexp_substr(JOB_NAME, '\\d+')) "; 
 
  mapping params;
  params[":RdbSource"]=sRdbSource; 
  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, dsExInfo);

  return sqlResult; 
}

/**
* Execute SQL query to get information about the latest session
*
* @param   sJobNumber        input, job number as a string, e.g.: "7"
* @param   sSessionId        output, id of the last session 
* @param   sSessionType      output, Type of the last session 
* @param   sSessionStartTime output, start time of the last session
* @param   sSessionEndTime   output, end time of the last session
* @param   dsOpType          output, list of operation types
* @param   dfOpTime          output, list of operation times
* @param   dulOpNoOfRecs     output, list of number of records
* @return                    output, true=success,false=error 
*/
public bool unLHCLoggingDBSql_getSessionInfo(const string sJobNumber, 
                                       string &sSessionId, string &sSessionType, 
                                       string &sSessionStartTime, string &sSessionEndTime, 
                                       dyn_string &dsOpType, dyn_float &dfOpTime, dyn_ulong &dulOpNoOfRecs,
                                       bool bUTC=false)
{
  string sql;  
  sql = "select as_.session_id, as_.session_type, " + 
        " to_char(" + ( bUTC==true ? "as_.session_start_utc" : "from_tz(as_.session_start_utc, 'UTC') at local" ) + ", 'YYYY/MM/DD HH24:MI:SS'), "+
        " to_char(" + ( bUTC==true ? "as_.session_end_utc" : "from_tz(as_.session_end_utc, 'UTC') at local" ) + ",'YYYY/MM/DD HH24:MI:SS'), "+
        " ao.op_type, ao.time_in_s, ao.no_of_recs from ( "+
        "  select * from ( " + 
        "   select * from RDB2LHC.audit_session where session_type='LDB_DATA_TRANSFER_" + sJobNumber + "' order by session_id desc) where rownum <=1) as_ " +
        "    left outer join RDB2LHC.audit_operation ao on as_.session_id=ao.session_id " +
        " order by ao.op_type asc" ;  // first collect then log 
    
  dyn_string dsExInfo;  
  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo);  
  if(dynlen(dsExInfo)>0) {
    return false;
  } 
  
  dynClear(dsOpType);
  dynClear(dfOpTime);
  dynClear(dulOpNoOfRecs);
  
  for(int i=1; i<=dynlen(sqlResult); i++)
  {
    if(i==1) {
      sSessionId = sqlResult[i][1];
      sSessionType = sqlResult[i][2]; 
      sSessionStartTime = sqlResult[i][3];
      sSessionEndTime = sqlResult[i][4]; 
    }
    if(sqlResult[i][5] != "") {
      dsOpType[i] = sqlResult[i][5];
      dfOpTime[i] = sqlResult[i][6];
      dulOpNoOfRecs[i] = sqlResult[i][7];  
    }  
  }   
  
  return true;  
}

/**
* Get all data categories related to a DB-schema
*
* @param sDbSchema    input, list of user names
* @param dsExInfo     output, Exeption information
*
* @return             output, list of data categories
*/
public dyn_string unLHCLoggingDBSql_getDataCategoriesOfSchema(string sDbSchema, dyn_string &dsExInfo)
{  
  string sql = "SELECT DISTINCT ( mvc.data_category_name || '/' || mvc.data_category_sub_name ) " + 
               " FROM rdb2lhc.meta_variables mv " +
               " INNER JOIN rdb2lhc.meta_variable_categories mvc ON mv.data_category_id = mvc.data_category_id " +
               "WHERE mv.rdb_source LIKE '"+sDbSchema+".%'";

  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo, true);  
  return sqlResult[1];
}

/**
* Get data category id of a data category
*
* @param sDataCategory  input, "data_category_name/data_category_sub_name"
* @param dsExInfo       output, Exeption information
*
* @return               output, data category id as a string
*/
public string unLHCLoggingDBSql_getDataCategoryId(string sDataCategory, dyn_string &dsExInfo)
{ 
  string sql = "SELECT data_category_id " + 
               " FROM rdb2lhc.meta_variable_categories " +
               "WHERE (data_category_name || '/' || data_category_sub_name) = :cat ";
  
  mapping params;
  params[":cat"]=sDataCategory;

  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryDataWithParMapping(sql, params, dsExInfo);  
  if(dynlen(sqlResult)==1) return (sqlResult[1][1]);
  else return ("");
}

/**
* Unregister single variable from logging
*
* @param sVariableName    input, variable name
* @return                 output, true: success, false: error
*/
public bool unLHCLoggingDBSql_unregisterVariableFromLogging(const string sVariableName)
{
  mapping params;
  params[":varName"] = sVariableName;
  
  string sql = "BEGIN pvss_data_transfer_mgr.mark_var_obsolete(UPPER(:varName)); END;" ;  
  
  if(unLHCLoggingDBSql_executeWithParMapping(sql, params) == false) {
    return false;  
  }    
  return true;  
}

/**
* Find inconsistency between registered data (meta_variables) and dpes (elements)
*
* @param sDbSchema    input, name of the db schema to check
* @param dsExInfo     output, Exeption information
*
* @return             output, list of inconsistent variables
*
* sqlResult[x][1]: variable_id
* sqlResult[x][2]: variable_name
*/
public dyn_dyn_mixed unLHCLoggingDBSql_getInconsistentData(const string sDbSchema, dyn_string &dsExInfo)
{
  string sql = "SELECT mv.variable_id, mv.variable_name FROM rdb2lhc.meta_variables mv " +
               " LEFT JOIN "+sDbSchema+".elements el ON mv.element_id = el.element_id " +
               "WHERE mv.rdb_source LIKE '"+sDbSchema+".%' " +
               " AND mv.obsolete_since_utc IS NULL " +
               " AND el.element_id IS NULL "; 

  dyn_dyn_mixed sqlResult = unLHCLoggingDBSql_queryData(sql, dsExInfo);

  return sqlResult;  
}
