/**@name LIBRARY: unSystemIntegrity_LHCLoggingDB.ctl

@author: 
Josef Hofer (EN-ICE-SCD)

Modification History: 
  23/06/2014: Josef Hofer
  - creation
  
Creation Date: 23/06/2014

version 1.0
	
Purpose: 
 System integrity plugin for the unLHCLoggingDB component.
	 	
Usage: Public

PVSS manager usage: CTRL (WCCOACTRL)

*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------

// String constants for adressing datapoints
const string c_SysInt_LHCLoggingDB = "LHCLoggingDB";
const string LHCLoggingDB_pattern = "LHCLoggingDB";

// Indexes of settings in .config.data
const int c_SysInt_LHCLoggingDB_ConfigIndex_Count                 = 6;
const int c_SysInt_LHCLoggingDB_ConfigIndex_LoopDelay             = 1;
const int c_SysInt_LHCLoggingDB_ConfigIndex_MaxAgeTransferJobs    = 2;
const int c_SysInt_LHCLoggingDB_ConfigIndex_MaxAgeTransferHistory = 3;
const int c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationUser      = 4;
const int c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationAttempts  = 5;
const int c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationWaitTime  = 6;

// Default values for .config.data
const unsigned c_SysInt_LHCLoggingDB_DefaultConfig_LoopDelay             = 240;  // [s] 
const unsigned c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferJobs    = 360;  // [s]  
const unsigned c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferHistory = 360;  // [s]  
const string   c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationUser      = "SystemIntegrity";  
const unsigned c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationAttempts  = 5;    
const unsigned c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationWaitTime  = 3;    // [s] 

// Default values for hard coded settings

// - no hard coded settings at the moment

// Message indexes in LHCLoggingDB catalogue
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorNotFound             = 1;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorLoadDbSettings       = 2;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorOpenDbConnection     = 3;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorDbSchemaNotFound     = 4;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorCreatingTestDps      = 5;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorDeletingTestDps      = 6;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorRegisteringTestDps   = 7;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorUnregisteringTestDps = 8;
const int c_SysInt_LHCLoggingDB_Catalogue_ConfigDataInconsistent    = 9;
const int c_SysInt_LHCLoggingDB_Catalogue_ConfigDataHasBeenReset    = 10;
const int c_SysInt_LHCLoggingDB_Catalogue_ErrorStoppingThread       = 11;

// TODO add additional messages

// Integrity classes

const string c_SysInt_LHCLoggingDB_IntegrityClass_TransferJobs     = "LHCLoggingDB_TransferJobs";
const string c_SysInt_LHCLoggingDB_IntegrityClass_TransferHistory  = "LHCLoggingDB_TransferHistory";
const string c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints   = "LHCLoggingDB_TestDatapoints";
const string c_SysInt_LHCLoggingDB_IntegrityClass_DataConsistency  = "LHCLoggingDB_DataConsistency";


// Alert levels of the Integrity classes

const unsigned c_SysInt_LHCLoggingDB_TransferJobs_ErrorQuery            = 70; 
const unsigned c_SysInt_LHCLoggingDB_TransferJobs_ErrorLagging          = 20; 
const unsigned c_SysInt_LHCLoggingDB_TransferJobs_ErrorDisabled         = 10;
const unsigned c_SysInt_LHCLoggingDB_TransferJobs_OK                    =  0;  

const unsigned c_SysInt_LHCLoggingDB_TransferHistory_ErrorQuery         = 70;  
const unsigned c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration  = 30;  
const unsigned c_SysInt_LHCLoggingDB_TransferHistory_ErrorTransfer      = 20;  
const unsigned c_SysInt_LHCLoggingDB_TransferHistory_WarningLagging     = 10;  
const unsigned c_SysInt_LHCLoggingDB_TransferHistory_OK                 =  0;

const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_ErrorQuery          = 70; 
const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration   = 40; 
const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_ErrorTransfer       = 30; 
const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_Retransfering       = 20; 
const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_Initialising        = 10; 
const unsigned c_SysInt_LHCLoggingDB_TestDatapoints_OK                  =  0;

const unsigned c_SysInt_LHCLoggingDB_DataConsistency_ErrorQuery         = 70; 
const unsigned c_SysInt_LHCLoggingDB_DataConsistency_Inconsistent       = 10;  
const unsigned c_SysInt_LHCLoggingDB_DataConsistency_OK                 =  0;

//--------------------------------------------------------------------------------------------------------------------------------
// declaration of global variables
//------------------

global dyn_string g_SysInt_LHCLoggingDB_ActiveIntegrityClasses;   // only needed for panel to show activated integrity classes   
global string g_SysInt_LHCLoggingDB_mySystemDbSchema;             // db schema of this system
global int g_SysInt_LHCLoggingDB_monitoringThreadId;              // ID of the monitoring thread

global unsigned g_SysInt_LHCLoggingDB_loopDelay;                  // [s] run checks according to this value
global unsigned g_SysInt_LHCLoggingDB_maxAgeTransferJobs;         // [s] transfer jobs have to run within this time 
global unsigned g_SysInt_LHCLoggingDB_maxAgeTransferHistory;      // [s] transfer history has to be updated within this time
global string   g_SysInt_LHCLoggingDB_registrationUser;           // username for registering test datapoints
global unsigned g_SysInt_LHCLoggingDB_registrationAttempts;       // maximum attempts to read ELEMENTS table before registering     
global unsigned g_SysInt_LHCLoggingDB_registrationWaitTime;       // wait time after unsucessfull attempt to read ELEMENTS table before doing next attempt

//@{

//---------------------------------------------------------------------------------------------------------------------------------


// unSystemIntegrity_LHCLoggingDB_getAvailableIntegrityClasses
/**
* Return all available integrity classes in a dyn_string
*
* @return     output, list of integrity classes 
*/
public dyn_string unSystemIntegrity_LHCLoggingDB_getAvailableIntegrityClasses()
{
  return makeDynString(c_SysInt_LHCLoggingDB_IntegrityClass_TransferJobs,
                       c_SysInt_LHCLoggingDB_IntegrityClass_TransferHistory,
                       c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints,
                       c_SysInt_LHCLoggingDB_IntegrityClass_DataConsistency);
}

//---------------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_getIntegrityClassDescription
/**
* Return the discription of a specified integrity class as a string
*
* @return     output, description string of integrity class 
*/
public string unSystemIntegrity_LHCLoggingDB_getIntegrityClassDescription(string sIntegrityClass) 
{
  switch(sIntegrityClass) 
  {
    case c_SysInt_LHCLoggingDB_IntegrityClass_TransferJobs: 
      return "Check current status of all transfer jobs related to this project and trigger alarm if they are not running.";
      
    case c_SysInt_LHCLoggingDB_IntegrityClass_TransferHistory: 
      return "Check the latest data transfer from RDBArchive to LDB for all registered elements and trigger alarm if errors are detected.";
      
    case c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints: 
      return "Check transfer history for test datapoints and trigger alarm if test values were not transferred properly. " +
             "The latest 3 transfers within an hour are checked and an error will be triggered if timestamps do not increase. " +
             "Recovering to state OK after an error will at least take 15 minutes.";
      
    case c_SysInt_LHCLoggingDB_IntegrityClass_DataConsistency: 
      return "Check data consistency between all elements registered for logging and all datapoints with activated archiving. " +
             "An alarm is triggered, if inconsistencies were found.";  
      
    default: 
      return "";      
  }
}

//---------------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_Initialize
/**
* Initialise system integrity plugin for LHCLoggingDB.
*
* @param dsResult   output, not used 
*/
public void unSystemIntegrity_LHCLoggingDB_Initialize(dyn_string &dsResult)
{
DebugTN("################### unSystemIntegrity_LHCLoggingDB_Initialize #####################");  

  // Get configuration from datapoint 
  unSystemIntegrity_LHCLoggingDB_getConfigData(true);

  // Initialise database
  unSystemIntegrity_LHCLoggingDB_initDabatase();
 
  // Look for active integrity classes
  dyn_string dsSysIntAlarmDps = dpNames(c_unSystemAlarm_dpPattern + LHCLoggingDB_pattern + "*", c_unSystemAlarm_dpType);
  for(int i=1; i<=dynlen(dsSysIntAlarmDps); i++) {
    string sAlarmDpToCheck = dpSubStr(dsSysIntAlarmDps[i], DPSUB_DP);
    dynAppend(g_SysInt_LHCLoggingDB_ActiveIntegrityClasses, sAlarmDpToCheck); 
  }
}

//---------------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_HandleCommand
/**
* Handle systemIntegrity commands from configuration panel
*
* @param sDpe1      input, dpe
* @param command    input, the systemIntegrity command
* @param sDpe2      input, dpe
* @param parameters input, the list of parameters of the systemIntegrity command
*/
public void unSystemIntegrity_LHCLoggingDB_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
  DebugTN("################### unSystemIntegrity_LHCLoggingDB_HandleCommand #####################");    
  DebugTN("Command: " + command + "; Parameters: " + parameters);  

  dyn_string dsExInfo;

	 switch(command) 
  {
	   case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
    {
      // Reinitialise database
      unSystemIntegrity_LHCLoggingDB_initDabatase();        
        
      for(int i=1; i<=dynlen(parameters); i++) { 
				    unSystemIntegrity_LHCLoggingDB_manageIntegrityClass(parameters[i], true, false, true, dsExInfo);    
      }      
      // ensure that thread is running
      unSystemIntegrity_LHCLoggingDB_manageMonitoringThread("start");    
			   break;
    }
    case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
    {       
      for(int i=1; i<=dynlen(parameters); i++) {  
				    unSystemIntegrity_LHCLoggingDB_manageIntegrityClass(parameters[i], false, true, false, dsExInfo);
      }      
      delay(1);         
      // stop thread if necessary
      unSystemIntegrity_LHCLoggingDB_manageMonitoringThread("stop");        
			   break;
    }       
    case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
    {
       // Reinitialise database
      unSystemIntegrity_LHCLoggingDB_initDabatase();            
        
      for(int i=1; i<=dynlen(parameters); i++) {  
				    unSystemIntegrity_LHCLoggingDB_manageIntegrityClass(parameters[i], false, false, true, dsExInfo);	
      }            
      // ensure that thread is running
      unSystemIntegrity_LHCLoggingDB_manageMonitoringThread("start");     
		    break;
    }    
		  case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
    {    
      for(int i=1; i<=dynlen(parameters); i++) {  
				    unSystemIntegrity_LHCLoggingDB_manageIntegrityClass(parameters[i], false, false, false, dsExInfo);
      }        
      delay(1);
      // stop thread if necessary
      unSystemIntegrity_LHCLoggingDB_manageMonitoringThread("stop");  
		    break;
    }   
    case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
    {
		    dpSet(c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION + ".diagnostic.commandResult", command, 
			         c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION + ".diagnostic.result", g_SysInt_LHCLoggingDB_ActiveIntegrityClasses);
		    break;
    }    
		  default:
    {
		    break;
    }
	 }

 	if(dynlen(dsExInfo) > 0) {
	   if(isFunctionDefined("unMessageText_sendException")) {
		    unMessageText_sendException("*", "*", g_SystemIntegrity_sSystemName+"unSystemIntegrity_LHCLoggingDB_HandleCommand", "user", "*", dsExInfo);
		  }
    // handle any error in case the send message failed
	   if(dynlen(dsExInfo) > 0) {
		    DebugTN(g_SystemIntegrity_sSystemName+"unSystemIntegrity_LHCLoggingDB_HandleCommand", dsExInfo);
	   }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_DataCallback
/**
* Callback function to get most recent configuration data
*
* @param sDPe       
* @param dsResult    
*/
public void unSystemIntegrity_LHCLoggingDB_DataCallback(string sDPe, dyn_string &dsResult)
{
  DebugTN("################### unSystemIntegrity_LHCLoggingDB_DataCallback #####################");   

  unSystemIntegrity_LHCLoggingDB_getConfigData(false);  
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_postInstall
/**
* Create systemIntegrity datapoint for LHCLoggingDB 
*/
public void unSystemIntegrity_LHCLoggingDB_postInstall()
{
  const string sSysIntDp = c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION;
  
  DebugTN("unSystemIntegrity_LHCLoggingDB_postInstall(): performing post installation steps");  
  
  if(!dpExists(sSysIntDp))
  {
    dpCreate(sSysIntDp, UN_SYSTEMINTEGRITY_DPTYPE);
    
    dpSetWait(       
      sSysIntDp + ".config.functions", 
        makeDynString("unSystemIntegrity_LHCLoggingDB_Initialize"
                     ,"unSystemIntegrity_LHCLoggingDB_HandleCommand"
                     ,"unSystemIntegrity_LHCLoggingDB_DataCallback"),        
      sSysIntDp + ".config.configurationPanel", 
        "vision/systemIntegrity/systemIntegrity_LHCLoggingDB_configuration.pnl",        
      sSysIntDp + ".config.operationPanel", 
        "vision/systemIntegrity/systemIntegrity_LHCLoggingDB_operation.pnl"             
    );    

    unSystemIntegrity_LHCLoggingDB_getConfigData(true);
  }
}   


//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_initDabatase
/**
* Get database settings, db schema and check if db schema exists in database
*/
private void unSystemIntegrity_LHCLoggingDB_initDabatase()
{
  string sDbName, sDbUser, sDbPass;  
  
  // Get database settings  
  if(unLHCLoggingDBSettings_getActiveDbConfig(sDbName, sDbUser, sDbPass)==false) {
    unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorLoadDbSettings);
    return; 
  }
  // Connect to database
  if (unLHCLoggingDBSql_openDbConnection(sDbName, sDbUser, sDbPass)==false) {
    unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorOpenDbConnection);
    return;
  } 
  
  // Get list DB schemas   
  dyn_string dsExInfo; 
  dyn_string dsDbSchemas = unLHCLoggingDBSql_getDbSchemas(dsExInfo); 

  // Get DB schema of this system 
  g_SysInt_LHCLoggingDB_mySystemDbSchema = unLHCLoggingDBSettings_getMySystemDbSchema();
  
  // Check if schema is know by the database
  if(dynContains(dsDbSchemas, g_SysInt_LHCLoggingDB_mySystemDbSchema) <= 0) {
    unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorDbSchemaNotFound);
    return;
  }   
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_createTestDPs
/**
* Create a test datapoint for each data category, if it does not exist yet
*/
private void unSystemIntegrity_LHCLoggingDB_createTestDPs()
{ 
  // Get all archive groups
  dyn_string dsArchiveClasses, dsArchiveGroupDps, dsExInfo;
  fwArchive_getAllRDBArchiveClasses(makeDynString(getSystemName()), dsArchiveClasses, dsArchiveGroupDps, dsExInfo);
  
  // Check if there is an archive group called "EVENT"
  string sArchiveClass="";
  for(int i=1;i<=dynlen(dsArchiveClasses);i++) {
    if(patternMatch("*EVENT*", dsArchiveClasses[i]) == true) {    
      sArchiveClass = dsArchiveClasses[i];
      break;
    }    
  }  
  if(sArchiveClass == "") {
    unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorCreatingTestDps, "Archive group 'EVENT' not found");
    return;
  }
  
  // Get data categories for which we will create test datapoints
  dyn_string dsDataCategories = unLHCLoggingDBSql_getDataCategoriesOfSchema(g_SysInt_LHCLoggingDB_mySystemDbSchema, dsExInfo);
  
  // Create test datapoints
  for(int i=1;i<=dynlen(dsDataCategories);i++)
  {
    string sTestDP = c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern + "_" + dsDataCategories[i];    
    if(!dpExists(sTestDP))
    {
      dpCreate(sTestDP, c_SysInt_RDB_TestDp_Type);
      if(!dpExists(sTestDP)) {
        unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorCreatingTestDps, "Datapoint " + sTestDP + " not created");        
        return;        
      }      
//    dpSetAlias(sTestDP + c_SysInt_RDB_TestDpe_Value, "TODO find a plausible alias");  // TODO find a naming schema for alias

      int iArchiveType = 0;
      dpGet(sTestDP + c_SysInt_RDB_TestDpe_Value + ":_" + "archive.._type", iArchiveType);    
      if(iArchiveType == 0) { 
        fwArchive_set(sTestDP + c_SysInt_RDB_TestDpe_Value, sArchiveClass, DPATTR_ARCH_PROC_VALARCH, 0, 0, 0, dsExInfo); 
        if(dynlen(dsExInfo)) {
          unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorCreatingTestDps, "Archive config not created for " + sTestDP + c_SysInt_RDB_TestDpe_Value);    
          return;
        }      
      }
    } 
  } 
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_registerTestDPs
/**
* Register test dapaoints to logging 
*/
private void unSystemIntegrity_LHCLoggingDB_registerTestDPs()
{
  dyn_string dsDpe, dsVarName, dsDescription, dsDataCategoryId, dsHierarchy;
  dyn_string dsElementId, dsUnit, dsRDBSource, dsErrors, dsWarnings;  
  dyn_string dsExInfo;
  
  dyn_string dsTestDP = dpNames(c_SysInt_RDB_TestDp_Prefix + "_"+LHCLoggingDB_pattern + "_" + "*", c_SysInt_RDB_TestDp_Type);
  
  for(int i=1;i<=dynlen(dsTestDP);i++) {
    string sDataCategory = substr(dsTestDP[i], strlen(getSystemName() + c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern + "_"));
    dynAppend(dsDataCategoryId, unLHCLoggingDBSql_getDataCategoryId(sDataCategory, dsExInfo));      
    strreplace(sDataCategory, "/", "_");  
    dynAppend(dsVarName,  c_SysInt_RDB_TestDp_Prefix + "." + LHCLoggingDB_pattern + "." + getSystemName() + sDataCategory);          
    dynAppend(dsDpe, dpSubStr(dsTestDP[i] + c_SysInt_RDB_TestDpe_Value, DPSUB_SYS_DP_EL));             
    dynAppend(dsDescription, "System integrity test datapoint for LHCLoggingDB");
    dynAppend(dsHierarchy, "");   
  }  
  
  if(dynlen(dsDpe) > 0) { 
    unsigned attempts=0;
    while(true) {
      if(unLHCLoggingDBParser_getElementData(dsDpe, dsElementId, dsUnit, dsRDBSource, dsErrors, dsWarnings, false) == true) break;
      if(++attempts > g_SysInt_LHCLoggingDB_registrationAttempts) {
        unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorRegisteringTestDps, "Error retrieving ELEMENT_ID");        
        return;
      }
      delay(g_SysInt_LHCLoggingDB_registrationWaitTime);  
    }
    
    string sUser = g_SysInt_LHCLoggingDB_registrationUser;
    string sPass = unLHCLoggingDBSql_getPassword(sUser);
    if(false == unLHCLoggingDBSql_registerDataForLoggingNew(dsElementId, dsVarName, dsRDBSource, dsUnit, dsDescription, dsDataCategoryId, dsHierarchy, sUser, sPass, false)) {
      unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorRegisteringTestDps, "Error during registration"); 
      return;
    }
  }  
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_deleteTestDPs
/**
* Delete test datapoints for each data category
*/
private void unSystemIntegrity_LHCLoggingDB_deleteTestDPs()
{ 
  dyn_string dsTestDP = dpNames(c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern + "_" + "*", c_SysInt_RDB_TestDp_Type);
  for(int i=1;i<=dynlen(dsTestDP);i++) {      
    dpDelete(dsTestDP[i]);    
    dyn_errClass err = getLastError();
    if(dynlen(err) > 0) {
      unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorDeletingTestDps); 
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_unregisterTestDPs
/**
* Unregister test dapaoints from logging 
*/
private void unSystemIntegrity_LHCLoggingDB_unregisterTestDPs()
{ 
  dyn_string dsTestDP = dpNames(c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern + "_" + "*", c_SysInt_RDB_TestDp_Type);
  for(int i=1;i<=dynlen(dsTestDP);i++) {    
    string sDataCategory = substr(dsTestDP[i], strlen(getSystemName()+c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern+"_"));
    strreplace(sDataCategory, "/", "_");  
    string sVarName = c_SysInt_RDB_TestDp_Prefix + "." + LHCLoggingDB_pattern + "." + getSystemName() + sDataCategory;      
    if (unLHCLoggingDBSql_unregisterVariableFromLogging(sVarName) == false) {
      unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorUnregisteringTestDps); 
      return;
    }    
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_getConfigData
/**
* Get configuration data and settings from RDB_systemIntegrityInfo
*
* @param   bResetOnError   input,  true: set default values on error, false: don't set default values on error
*/
private void unSystemIntegrity_LHCLoggingDB_getConfigData(bool bResetOnError)
{
  if(dpExists(c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION))
  {
    dyn_string dsConfigData;  
    dpGet(c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", dsConfigData);

    if(dynlen(dsConfigData) == c_SysInt_LHCLoggingDB_ConfigIndex_Count)
    {   
      g_SysInt_LHCLoggingDB_loopDelay             = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_LoopDelay]; 
      g_SysInt_LHCLoggingDB_maxAgeTransferJobs    = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_MaxAgeTransferJobs]; 
      g_SysInt_LHCLoggingDB_maxAgeTransferHistory = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_MaxAgeTransferHistory]; 
      g_SysInt_LHCLoggingDB_registrationUser      = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationUser]; 
      g_SysInt_LHCLoggingDB_registrationAttempts  = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationAttempts];   
      g_SysInt_LHCLoggingDB_registrationWaitTime  = dsConfigData[c_SysInt_LHCLoggingDB_ConfigIndex_RegistrationWaitTime];
    }
    else  
    {
      if(bResetOnError==true)
      {
        dpSet(c_SysInt_LHCLoggingDB + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", 
              makeDynString(c_SysInt_LHCLoggingDB_DefaultConfig_LoopDelay
                           ,c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferJobs
                           ,c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferHistory
                           ,c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationUser
                           ,c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationAttempts
                           ,c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationWaitTime)
             );        
        g_SysInt_LHCLoggingDB_loopDelay             = c_SysInt_LHCLoggingDB_DefaultConfig_LoopDelay; 
        g_SysInt_LHCLoggingDB_maxAgeTransferJobs    = c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferJobs;
        g_SysInt_LHCLoggingDB_maxAgeTransferHistory = c_SysInt_LHCLoggingDB_DefaultConfig_MaxAgeTransferHistory;
        g_SysInt_LHCLoggingDB_registrationUser      = c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationUser;
        g_SysInt_LHCLoggingDB_registrationAttempts  = c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationAttempts;
        g_SysInt_LHCLoggingDB_registrationWaitTime  = c_SysInt_LHCLoggingDB_DefaultConfig_RegistrationWaitTime;    
        
        unSystemIntegrity_LHCLoggingDB_throwError("INFO", c_SysInt_LHCLoggingDB_Catalogue_ConfigDataHasBeenReset);
      }
      else
      {
        unSystemIntegrity_LHCLoggingDB_throwError("WARNING", c_SysInt_LHCLoggingDB_Catalogue_ConfigDataInconsistent);
      }
    } 
  } 
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_manageIntegrityClass
/**
* This function is used to create/destruct/enable/disable alarm datapoints
* It can also create/destruct the test datapoints
*
* @param sDp             input, true create the dp and the alarm config if it does not exist, enable it
* @param bCreate         input, true create the dp and the alarm config if it does not exist, enable it
* @param bDelete         input, true create the dp and the alarm config if it does not exist, enable it
*	@param bRegister       input, true do a dpConnect, false do a dpDisconnect
*	@param exceptionInfo   output, exception are returned here
*/
private void unSystemIntegrity_LHCLoggingDB_manageIntegrityClass(string sDp, bool bCreate, bool bDelete, bool bEnable, dyn_string &exceptionInfo)
{
  // Remove the system name
  string sSysAlarmDp = unSystemIntegrity_removeSystemName(sDp);   
  
  if(bCreate) 
  {
    if(patternMatch("*"+c_SysInt_LHCLoggingDB_IntegrityClass_TransferJobs+"*", sSysAlarmDp) == true)
    {
      // create alarm datapoint     
      unSystemIntegrity_createSystemAlarmDiscrete(
          sSysAlarmDp, 
          "LHCLoggingDB transfer jobs:", 
          makeDynString("OK",                     
                        "Disabled transfer jobs found",
                        "Transfer job not executed in time",
                        "Query Error"), 
          makeDynString("*", 
                        c_SysInt_LHCLoggingDB_TransferJobs_ErrorDisabled, 
                        c_SysInt_LHCLoggingDB_TransferJobs_ErrorLagging,
                        c_SysInt_LHCLoggingDB_TransferJobs_ErrorQuery),  
          makeDynString("",
                        "_fwErrorNack.",   
                        "_fwErrorNack.",
                        "_fwErrorNack."),  
          exceptionInfo 
        );      
    }
    else if(patternMatch("*"+c_SysInt_LHCLoggingDB_IntegrityClass_TransferHistory+"*", sSysAlarmDp) == true)
    {
      // create alarm datapoint     
      unSystemIntegrity_createSystemAlarmDiscrete(
          sSysAlarmDp, 
          "LHCLoggingDB transfer history:", 
          makeDynString("OK",                     
                        "Transfer lagging",
                        "Transfer Error",
                        "Registration Error",
                        "Query Error"), 
          makeDynString("*", 
                        c_SysInt_LHCLoggingDB_TransferHistory_WarningLagging, 
                        c_SysInt_LHCLoggingDB_TransferHistory_ErrorTransfer, 
                        c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration,
                        c_SysInt_LHCLoggingDB_TransferHistory_ErrorQuery),  
          makeDynString("",
                        "_fwWarningNack.", 
                        "_fwErrorNack.",   
                        "_fwErrorNack.",
                        "_fwErrorNack."),
          exceptionInfo 
        );       
    } 
    
    else if(patternMatch("*"+c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints+"*", sSysAlarmDp) == true)
    { 
      // create alarm datapoint     
      unSystemIntegrity_createSystemAlarmDiscrete(
          sSysAlarmDp, 
          "LHCLoggingDB test datapoints:", 
          makeDynString("OK",                        
                        "Initialising",       
                        "Retransfer started", 
                        "Transfer Error",     
                        "Registration Error",                        
                        "Query Error"),         
          makeDynString("*", 
                        c_SysInt_LHCLoggingDB_TestDatapoints_Initialising, 
                        c_SysInt_LHCLoggingDB_TestDatapoints_Retransfering, 
                        c_SysInt_LHCLoggingDB_TestDatapoints_ErrorTransfer, 
                        c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration,                         
                        c_SysInt_LHCLoggingDB_TestDatapoints_ErrorQuery),  
          makeDynString("",
                        "_fwWarningNack.", 
                        "_fwWarningNack.", 
                        "_fwErrorNack.",   
                        "_fwErrorNack.",   
                        "_fwErrorNack."),  
          exceptionInfo 
        );        
  
      // create and register test datapoints  
      unSystemIntegrity_LHCLoggingDB_createTestDPs(); 
      unSystemIntegrity_LHCLoggingDB_registerTestDPs();
    }  
    else if(patternMatch("*"+c_SysInt_LHCLoggingDB_IntegrityClass_DataConsistency+"*", sSysAlarmDp) == true)
    {
      // create alarm datapoint     
      unSystemIntegrity_createSystemAlarmDiscrete(
          sSysAlarmDp, 
          "LHCLoggingDB data consistency: ", 
          makeDynString("OK",                   
                        "Inconsistencies found",
                        "Query Error"),  
          makeDynString("*",                    
                        c_SysInt_LHCLoggingDB_DataConsistency_Inconsistent,
                        c_SysInt_LHCLoggingDB_TestDatapoints_ErrorQuery),  
          makeDynString("",
                        "_fwErrorNack.",
                        "_fwErrorNack."),
          exceptionInfo 
        );       
    }
    
    if(dynlen(exceptionInfo)<=0) {  
      // add to active integrity classes
      _unSystemIntegrity_setList(g_SysInt_LHCLoggingDB_ActiveIntegrityClasses, sSysAlarmDp, true);     
    }
  } 
 
   // Do enabling and masking
  if(dynlen(exceptionInfo)<=0) 
  {     
    if(dpExists(sSysAlarmDp)) 
    {
				  dpSet(sSysAlarmDp + ".enabled", bEnable);
      unAlarmConfig_mask(sSysAlarmDp+".alarm", !bEnable, exceptionInfo);
    }    
  } 
 
  if(bDelete) 
  {  
    // remove dp from the alert list of applicationDP
  		_unSystemIntegrity_modifyApplicationAlertList(sSysAlarmDp, false, exceptionInfo);    
    
    // remove alert class from list 
    _unSystemIntegrity_setList(g_SysInt_LHCLoggingDB_ActiveIntegrityClasses, sSysAlarmDp, false);     
    
  		// delete alarm datapoint
    if(dpExists(sSysAlarmDp)) {
				  dpDelete(sSysAlarmDp);
    }  
  
    // unregister and delete test datapoints
    if(patternMatch("*"+c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints+"*", sSysAlarmDp) == true) {
      unSystemIntegrity_LHCLoggingDB_unregisterTestDPs();
      unSystemIntegrity_LHCLoggingDB_deleteTestDPs();  
    }    
  }  
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_manageMonitoringThread
/**
* This function is used to handle the monitorig thread which does the checking
*
* @param command         input, string: "start" or "stop" the thread
*/
private void unSystemIntegrity_LHCLoggingDB_manageMonitoringThread(string command)
{
  switch (command)
  {
    case "start":  
    {   
      // start only if thread is not running yet   
      if(g_SysInt_LHCLoggingDB_monitoringThreadId == 0) 
      {
        g_SysInt_LHCLoggingDB_monitoringThreadId = startThread("unSystemIntegrity_LHCLoggingDB_checkLogging");  
      }        
      break;
    }
    case "stop":
    {
      // stop thread only if necessary
      if(g_SysInt_LHCLoggingDB_monitoringThreadId != 0) 
      {   
        bool stop = true;         
        dyn_string dsSysIntAlarmDps = dpNames(c_unSystemAlarm_dpPattern + LHCLoggingDB_pattern + "*", c_unSystemAlarm_dpType);     
        for(int i=1;i<=dynlen(dsSysIntAlarmDps);i++) 
        {
          bool enable;
          dpGet(dsSysIntAlarmDps[i] + ".enabled", enable);
          if(enable == true) 
          {
            stop = false;     
          }         
        }        
        if(stop) 
        {
          if (stopThread(g_SysInt_LHCLoggingDB_monitoringThreadId) < 0) 
          {
            unSystemIntegrity_LHCLoggingDB_throwError("ERROR", c_SysInt_LHCLoggingDB_Catalogue_ErrorStoppingThread);  
          } 
          else 
          { 
            g_SysInt_LHCLoggingDB_monitoringThreadId = 0;
          }            
        } 
      }      
      break;
    }    
  } 
} 

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_checkLogging
/**
* This is the function that actually does the system integrity checking in a periodic cycle
*
* Following checks are done:
* ----------------------------------------------------------
* 1. unSystemIntegrity_LHCLoggingDB_checkTransferJobs
* 2. unSystemIntegrity_LHCLoggingDB_checkTransferHistory
* 3. unSystemIntegrity_LHCLoggingDB_checkTestDatapoints
* 4. unSystemIntegrity_LHCLoggingDB_checkDataConsistency
*
* The result of every check is written to a dedicated alarm datapoint
* Checks are only executed if the alarm datapoint is enabled
*/
private void unSystemIntegrity_LHCLoggingDB_checkLogging()
{  
  string sAlarmDp;
  bool bAlarmEnabled;  
  unsigned uResult;
  
  while(1)
  {      
    int iErrorCode = 0;      

    // 1. check status of transfer jobs 
    sAlarmDp = c_unSystemAlarm_dpPattern + c_SysInt_LHCLoggingDB_IntegrityClass_TransferJobs;        
    if(dpExists(sAlarmDp)) {      
      dpGet(sAlarmDp + ".enabled", bAlarmEnabled);
      if(bAlarmEnabled) {      
        uResult = unSystemIntegrity_LHCLoggingDB_checkTransferJobs(g_SysInt_LHCLoggingDB_mySystemDbSchema); 
        dpSetWait(sAlarmDp + ".alarm", uResult); 
      }   
    } 
    
    // 2. check transfer history of all registered elements
    sAlarmDp = c_unSystemAlarm_dpPattern + c_SysInt_LHCLoggingDB_IntegrityClass_TransferHistory;        
    if(dpExists(sAlarmDp)) {  
      dpGet(sAlarmDp + ".enabled", bAlarmEnabled);
      if(bAlarmEnabled) {      
        uResult = unSystemIntegrity_LHCLoggingDB_checkTransferHistory(g_SysInt_LHCLoggingDB_mySystemDbSchema);
        dpSetWait(sAlarmDp + ".alarm", uResult); 
      }  
    }   

    // 3. check transfer history of test datapoints   
    sAlarmDp = c_unSystemAlarm_dpPattern + c_SysInt_LHCLoggingDB_IntegrityClass_TestDatapoints;        
    if(dpExists(sAlarmDp)) {  
      dpGet(sAlarmDp + ".enabled", bAlarmEnabled);
      if(bAlarmEnabled) {     
        uResult = unSystemIntegrity_LHCLoggingDB_checkTestDatapoints(); 
        dpSetWait(sAlarmDp + ".alarm", uResult);       
      }  
    }  
    
    // 4. Check if all registered elements have an entry in ELEMENTS table
    sAlarmDp = c_unSystemAlarm_dpPattern + c_SysInt_LHCLoggingDB_IntegrityClass_DataConsistency;        
    if(dpExists(sAlarmDp)) {  
      dpGet(sAlarmDp + ".enabled", bAlarmEnabled);
      if(bAlarmEnabled) {      
        uResult = unSystemIntegrity_LHCLoggingDB_checkDataConsistency(g_SysInt_LHCLoggingDB_mySystemDbSchema);
        dpSetWait(sAlarmDp + ".alarm", uResult); 
      }  
    } 
    
    delay(g_SysInt_LHCLoggingDB_loopDelay);     
  } 
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_checkTransferJobs
/**
* Check current status for all transfer jobs related to sDbSchema
*
* @param sDbSchema        input, database schema for which we want to check the transfer jobs
*
* @return uReturnCode     output, Statuscode 
*/
public unsigned unSystemIntegrity_LHCLoggingDB_checkTransferJobs(string sDbSchema)
{
  unsigned uReturnCode = c_SysInt_LHCLoggingDB_TransferJobs_OK;
  
  dyn_string dsExInfo;
  
  time tDbSysTimeUtc = unLHCLoggingDBSql_getDbSysTimeUtc(dsExInfo);
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TransferJobs_ErrorQuery;  
  
  dyn_dyn_mixed ddmJobStatus = unLHCLoggingDBSql_getTransferJobStatus(sDbSchema+".%", dsExInfo);
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TransferJobs_ErrorQuery;

  for(int i=1;i<=dynlen(ddmJobStatus);i++) 
  {    
    if(ddmJobStatus[i][2] != "TRUE") {      
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferJobs_ErrorDisabled) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferJobs_ErrorDisabled;
      continue;
    } 
    
    time tLastJobStart = ddmJobStatus[i][4];      
    if(tDbSysTimeUtc-tLastJobStart > g_SysInt_LHCLoggingDB_maxAgeTransferJobs) {      
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferJobs_ErrorLagging) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferJobs_ErrorLagging;   
      continue;    
    }   
  }   
  
  return uReturnCode; 
}

//------------------------------------------------------------------------------------------------------------------------ 

// unSystemIntegrity_LHCLoggingDB_checkTransferHistory
/**
* Check current status of all log variables from a specific database schema
*
* @param sDbSchema       input, DB schema to check
*
* @return uReturnCode    output, Statuscode with highest priority
*/
public unsigned unSystemIntegrity_LHCLoggingDB_checkTransferHistory(const string sDbSchema)
{ 
  dyn_string dsExInfo;  
  unsigned uReturnCode = c_SysInt_LHCLoggingDB_TransferHistory_OK;

  time tDbSysTimeUtc = unLHCLoggingDBSql_getDbSysTimeUtc(dsExInfo);
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TransferHistory_ErrorQuery;  
  
  dyn_dyn_mixed ddmLvsInfo = unLHCLoggingDBSql_getLvsInfo(sDbSchema+".%",dsExInfo); 
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TransferHistory_ErrorQuery;  
  
  for (int i=1;i<=dynlen(ddmLvsInfo);i++) 
  {     
    // Ensure that VARIABLE_NAME has a CREATED_UTC_STAMP, if not return ERROR
    if(ddmLvsInfo[i][2]=="") {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration;
      continue;    
    }
 
    // Ensure that all CREATED_UTC_STAMP are up to date (not older than TIME_DIFF_MAX), if not return ERROR
    time tCreatedUtc = ddmLvsInfo[i][2];     
    if(tDbSysTimeUtc-tCreatedUtc > g_SysInt_LHCLoggingDB_maxAgeTransferHistory) {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferHistory_ErrorTransfer) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferHistory_ErrorTransfer;
      continue;  
    }

    // Ensure that VARIABLE_NAME has a NO_RECS_LOGGED and MAX_ROWS, if not return ERROR
    if(ddmLvsInfo[i][4]=="" || ddmLvsInfo[i][5]=="") {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferHistory_ErrorRegistration;
      continue;
    }    

    // Ensure that data transfer is not lagging
    unsigned uNoRecsLogged = ddmLvsInfo[i][4];
    unsigned uNoRecsMax = ddmLvsInfo[i][5];    
    if(uNoRecsLogged==uNoRecsMax) {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TransferHistory_WarningLagging) 
        uReturnCode = c_SysInt_LHCLoggingDB_TransferHistory_WarningLagging;
      continue;
    }  
  } 
  return uReturnCode;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_checkTestDatapoints
/**
* Check current status for all test datapoints
*
* @return uReturnCode     output, Statuscode 
*/
public unsigned unSystemIntegrity_LHCLoggingDB_checkTestDatapoints()
{
  // Get variable names from dp names
  dyn_string dsVariableName;   
  dyn_string dsTestDp = dpNames(c_SysInt_RDB_TestDp_Prefix+"_"+LHCLoggingDB_pattern+"_"+"*", c_SysInt_RDB_TestDp_Type);    
  for(int i=1;i<=dynlen(dsTestDp);i++) {
    string sDataCategory = substr(dsTestDp[i], strlen(getSystemName() + c_SysInt_RDB_TestDp_Prefix + "_" + LHCLoggingDB_pattern + "_"));
    strreplace(sDataCategory, "/", "_");  // because in DB we don't use '/' for separation
    dynAppend(dsVariableName, c_SysInt_RDB_TestDp_Prefix + "." + LHCLoggingDB_pattern + "." + getSystemName() + sDataCategory);     
  }  
   
  dyn_string dsExInfo;
  unsigned uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_OK;        
  
  time tDbSysTimeUtc = unLHCLoggingDBSql_getDbSysTimeUtc(dsExInfo);   
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TestDatapoints_ErrorQuery;

  for(int i=1;i<=dynlen(dsVariableName);i++) 
  {  
    dyn_dyn_mixed ddmTestDpLvsData = unLHCLoggingDBSql_getLvsForTestDpe(dsVariableName[i], dsExInfo); 
    if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_TestDatapoints_ErrorQuery;
  
    // Do we have 3 results ? 
    if(dynlen(ddmTestDpLvsData)!=3) {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_Initialising) 
        uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_Initialising;
      continue;
    }

    // Do all results have an LAST_LOGGED_UTC timestamp ?
    if(ddmTestDpLvsData[1][1]=="" || ddmTestDpLvsData[2][1]=="" || ddmTestDpLvsData[3][1]=="" ||
       ddmTestDpLvsData[1][2]=="" || ddmTestDpLvsData[2][2]=="" || ddmTestDpLvsData[3][2]=="") 
    {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration) 
        uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration;
      continue; 
    } 
   
    time tCreatedUtc = ddmTestDpLvsData[1][1];     
    if(tDbSysTimeUtc-tCreatedUtc > g_SysInt_LHCLoggingDB_maxAgeTransferHistory) { 
      if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration) 
        uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_ErrorRegistration;
      continue; 
    }
  
    time t1 = ddmTestDpLvsData[1][2]; 
    time t2 = ddmTestDpLvsData[2][2]; 
    time t3 = ddmTestDpLvsData[3][2];     
  
    // Are the times increasing ?
    if(t1>t2 && t2>t3 && t1>t3) {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_OK) 
        uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_OK;
      continue; 
    }  
  
    // Are the times at least changing ?
    if(t1!=t2 && t2!=t3 && t1!=t3) {
      if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_Retransfering) 
        uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_Retransfering;
      continue; 
    }
    
    if (uReturnCode < c_SysInt_LHCLoggingDB_TestDatapoints_ErrorTransfer)
      uReturnCode = c_SysInt_LHCLoggingDB_TestDatapoints_ErrorTransfer;
  }
  
  return uReturnCode;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_checkDataConsistency
/**
* Check data consistecy between registered variables in database (table: META_VARIABLES) 
* and datapoints with an archive config in the PVSS project
*
* @param sDbSchema       input, DB schema to check
*
* @return uReturnCode    output, Statuscode 
*/
public unsigned unSystemIntegrity_LHCLoggingDB_checkDataConsistency(string sDbSchema)
{ 
  dyn_string dsExInfo;
  
  dyn_dyn_mixed ddmData = unLHCLoggingDBSql_getInconsistentData(sDbSchema, dsExInfo);  
  
  if(dynlen(dsExInfo)>0) return c_SysInt_LHCLoggingDB_DataConsistency_ErrorQuery;
  if(dynlen(ddmData)==0) return c_SysInt_LHCLoggingDB_DataConsistency_OK;   
  return c_SysInt_LHCLoggingDB_DataConsistency_Inconsistent;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_LHCLoggingDB_throwError
/**
* Write special error to message log
*
* @param severity     input, severity of the message
* @param code         input, error code to write
* @param sDetails     input, message details
*/
private void unSystemIntegrity_LHCLoggingDB_throwError(string severity = "ERROR", int code = 1, string sDetails="")
{
  int prio = PRIO_WARNING; //seems that throwing a fatal error makes Pmon stop monitoring this manager????
  int type = ERR_CONTROL;
  
  switch(strtoupper(severity))
  {
    case "INFO":    prio=PRIO_INFO;    break;
    case "WARNING": prio=PRIO_WARNING; break;
    case "ERROR":   prio=PRIO_SEVERE;  break;
  }
  
  errClass err = makeError("unSystemIntegrity_LHCLoggingDB", prio, type, code, sDetails);
  throwError(err);
  
  return;
}

